<?php

namespace Inventory\Item\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Inventory\Item\Http\Requests\ItemRequest;
use Inventory\Item\Interfaces\ItemRepositoryInterface;
use Inventory\Item\Models\Item;
use Inventory\Item\Forms\Item as Form;

/**
 * APIController  class for item.
 */
class ItemAPIController extends BaseController
{

    /**
     * Initialize item resource controller.
     *
     * @param type ItemRepositoryInterface $item
     *
     * @return null
     */
    public function __construct(ItemRepositoryInterface $item)
    {
        parent::__construct();
        $this->repository = $item;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Inventory\Item\Repositories\Criteria\ItemResourceCriteria::class);
    }

    /**
     * Display a list of item.
     *
     * @return Response
     */
    public function index(ItemRequest $request)
    {
        return $this->repository
            ->setPresenter(\Inventory\Item\Repositories\Presenter\ItemPresenter::class)
            ->paginate();
    }

    /**
     * Display item.
     *
     * @param Request $request
     * @param Model   $item
     *
     * @return Response
     */
    public function show(ItemRequest $request, Item $item)
    {
        return $item->setPresenter(\Inventory\Item\Repositories\Presenter\ItemListPresenter::class);
        ;
    }

    /**
     * Create new item.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ItemRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('item::item.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('item/item/' . $item->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('item/item');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the item.
     *
     * @param Request $request
     * @param Model   $item
     *
     * @return Response
     */
    public function update(ItemRequest $request, Item $item)
    {
        try {
            $data = $request->all();

            $item->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('item::item.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('item/item/' . $item->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('item/item/' . $item->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the item.
     *
     * @param Model   $item
     *
     * @return Response
     */
    public function destroy(ItemRequest $request, Item $item)
    {
        try {
            $item->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('item::item.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('item/item/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('item/item/' . $item->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
