<?php

namespace Inventory\Item\Providers;

use Illuminate\Support\ServiceProvider;

class ItemServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'item');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'item');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerItem();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerItem()
    {
        $this->app->bind('item', function($app) {
            return new Item($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Item', 'Lavalite\Item\Facades\Item');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('inventory.item', function ($app) {
            return $this->app->make('Inventory\Item\Item');
        });

        // Bind Unit to repository
        $this->app->bind(
            'Inventory\Item\Interfaces\UnitRepositoryInterface',
            \Inventory\Item\Repositories\Eloquent\UnitRepository::class
        );        // Bind Item to repository
        $this->app->bind(
            'Inventory\Item\Interfaces\ItemRepositoryInterface',
            \Inventory\Item\Repositories\Eloquent\ItemRepository::class
        );        // Bind Category to repository
        $this->app->bind(
            'Inventory\Item\Interfaces\CategoryRepositoryInterface',
            \Inventory\Item\Repositories\Eloquent\CategoryRepository::class
        );

        $this->app->register(\Inventory\Item\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Inventory\Item\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and item's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'inventory.item'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeItem::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['inventory.item'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('inventory/item.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/item')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/item')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
